import greenfoot.*;  // (World, Actor, GreenfootImage, and Greenfoot)

import java.util.List;
import java.util.ArrayList;

/**
 * Wombat. A Wombat moves forward until it can't do so anymore, at
 * which point it turns left. If a wombat finds a leaf, it eats it.
 * 
 * @author Michael Kolling
 * @version 1.0.1
 * Modified by Mark Twomey...
 */
public class Wombat extends Actor
{
    private static final int EAST = 0;
    private static final int WEST = 1;
    private static final int NORTH = 2;
    private static final int SOUTH = 3;
    
    private static final int DIAGONALLY_LEFT = 4;
    private static final int DIAGONALLY_RIGHT = 5;

    private int direction;
    private int leavesEaten;
    
    private boolean alwaysTurnRandomly = true;
    private boolean alwaysTurnLeft = false;
    private boolean alwaysTurnDiagonally = false;
    
    public Wombat()
    {
        setDirection(EAST);
        leavesEaten = 0;
    }

    /**
     * Do whatever the wombat likes to do.
     */
    public void act()
    {
        if ( foundLeaf() ) {
            eatLeaf();
        }
        else if ( canMove() ) {
            move();
        }
        else {
            if ( alwaysTurnRandomly ) {
                turnRandomly(); 
            } else if (alwaysTurnLeft) {
                turnLeft();
            }  else if (alwaysTurnDiagonally) {
                turnDiagonallyLeftOrRight();
            }
        }
    }

    /**
     * Check whether there is a leaf in the same cell as we are.
     */
    public boolean foundLeaf()
    {
        Actor leaf = getOneObjectAtOffset(0, 0, Leaf.class);
        if(leaf != null) {
            return true;
        }
        else {
            return false;
        }
    }
    
    /**
     * Eat a leaf.
     */
    public void eatLeaf()
    {
        Actor leaf = getOneObjectAtOffset(0, 0, Leaf.class);
        if ( leaf != null ) {
            // eat the leaf...
            getWorld().removeObject(leaf);
            leavesEaten = leavesEaten + 1; 
        }
    }
    
    /**
     * Move one cell forward in the current direction.
     */
    public void move()
    {
        if (! canMove()) {
            return;
        }
        
        switch(direction) {
            case SOUTH :
                setLocation(getX(), getY() + 1);
                break;
            case EAST :
                setLocation(getX() + 1, getY());
                break;
            case NORTH :
                setLocation(getX(), getY() - 1);
                break;
            case WEST :
                setLocation(getX() - 1, getY());
                break;
        }
    }

    /**
     * Test if we can move forward. Return true if we can, false otherwise.
     */
    public boolean canMove()
    {
        World myWorld = getWorld();
        
        int checkPositionAcross = getX(); // current position from zero left 
        int checkPositionDown = getY();   // current position from zero top 
        
        switch(direction) {
            case SOUTH : // Down
                checkPositionDown++;
                break;
            case EAST :  // Right
                checkPositionAcross++;
                break;
            case NORTH : // Up
                checkPositionDown--;
                break;
            case WEST :  // Left
                checkPositionAcross--;
                break;
        }
        
        // Check if we are trying to walk outside of Wombat World
        if (checkPositionAcross >= myWorld.getWidth() || checkPositionDown >= myWorld.getHeight()) {
            return false;
        }
        else if (checkPositionAcross < 0 || checkPositionDown < 0) {
            return false;
        }
       
        
        // Check if there is a rock in the way...
        List rocks = myWorld.getObjectsAt(checkPositionAcross, checkPositionDown, Rock.class);
        if ( ! rocks.isEmpty() ) {     
            return false;
        }
        
        // If there are no problems then just move.
         return true;
    }

    /**
     * Turns towards the left.
     */
    public void turnLeft()
    {
        switch(direction) {
            case SOUTH :
                setDirection(EAST);
                break;
            case EAST :
                setDirection(NORTH);
                break;
            case NORTH :
                setDirection(WEST);
                break;
            case WEST :
                setDirection(SOUTH);
                break;
        }
    }
        
    
    /*
     * Turn in a random direction.... 
     *   by looping a random number of times (1-3)
     *   and each time we turn LEFT 
     *   (which we already know how to do)
     */
    public void turnRandomly()
    {
        int turns = Greenfoot.getRandomNumber(3);
        
        for (int turnNumber = 0; turnNumber < turns; turnNumber ++ ) {
            turnLeft();
        }
    }
    
    
    /*
     * Turn diagonally either LEFT 45 degrees or RIGHT 45 degrees (randomly)
     */
    public void turnDiagonallyLeftOrRight()
    {
        int diagonallyLeftOrRight = Greenfoot.getRandomNumber(2); // A random number, 0 or 1
        
        if (diagonallyLeftOrRight == 0) {
            setDirection(DIAGONALLY_LEFT);
        }  else {
            setDirection(DIAGONALLY_RIGHT);
        }
    }
    // ****************************************************************
    
    
    /**
     * Set the direction we're facing.
     * 
     * convert from the compass direction into degrees of rotation
     */
    public void setDirection(int direction)
    {
        this.direction = direction;
        switch(direction) {
            case SOUTH :
                setRotation(90);
                break;
            case EAST :
                setRotation(0);
                break;
            case NORTH :
                setRotation(270);
                break;
            case WEST :
                setRotation(180);
                break;
            case DIAGONALLY_LEFT :
                setRotation(getRotation() - 45);
                break;
            case DIAGONALLY_RIGHT :
                setRotation(getRotation() + 45);
                break;
            default :
                break;
        }
    }

    /**
     * Tell how many leaves we have eaten.
     */
    public int getLeavesEaten()
    {
        return leavesEaten;
    }
    
        
    public void askToalwaysTurnRandomly() {
        alwaysTurnRandomly = true;
        alwaysTurnLeft = false;
        alwaysTurnDiagonally = false;  
    }
    
    public void askToAlwaysTurnLeft() {
        alwaysTurnRandomly = false; 
        alwaysTurnLeft = true;
        alwaysTurnDiagonally = false;
    }
    
    public void askToalwaysTurndiagonally() {
        alwaysTurnRandomly = false;
        alwaysTurnLeft = false;
        alwaysTurnDiagonally = true;  
    }
}